AWS_ACCESS_KEY_PARAM = 'imco_aws_access_key_id'
AWS_SECRET_ACCESS_KEY_PARAM = 'imco_aws_secret_access_key'
AWS_CREDENTIAL_PARAMS = [AWS_ACCESS_KEY_PARAM, AWS_SECRET_ACCESS_KEY_PARAM]

DEPLOY_CLOUD_ACCOUNT = 'deploy_cloud_account'
IMCO_CUSTOMER_ID = 'imco_customer_id'
CONNECT_PARAM_CREDENTIALS_SOURCE = 'deploy_credentials_source'
CONNECT_PARAM_CREDENTIALS_SOURCE_INTEGRATED = 'repeat'
CONNECT_PARAM_CREDENTIALS_SOURCE_MANUAL = 'manual'

CONNECT_PARAM_CLOUD_ACCOUNT = 'deploy_cloud_account'
CONNECT_PARAM_IMCO_CUSTOMER_ID = 'imco_customer_id'
